/*#######################################################################################
AVR Small Webserver 

Copyright (C) 2004 Ulrich Radig

#######################################################################################*/

#include "main.h"
#include "rtl8019.h"
#include "arp.h"
#include "ethernet.h"

struct S_ARP_ENTRY arp_cache[ARP_TABLE_SIZE];
extern unsigned char packet[MTU_SIZE];

//############################################################################
//Diese Routine erzeugt ein ARP buffer
void ARP_Reply (char *buffer,int *bufferlen)
//############################################################################
{
	printf("ARP_Reply: ");
	struct Ethernet_Header *ethernet;
	ethernet = (struct Ethernet_Header *)&buffer[ETHER_OFFSET];

	struct ARP_Header *arp;
	arp = (struct ARP_Header *)&buffer[ARP_OFFSET];

	//2 Byte Hardware Typ: Enthlt den Code fr Ethernet
	if (arp->ARP_HWType == 0x0100 &&
		//2 Byte Protokoll Typ: Enthlt den Code fr IP
		arp->ARP_PRType == 0x0008 &&
		//1Byte Lnge der Hardwareadresse:Enthlt 6 fr 6 Byte MAC Addresse
		arp->ARP_HWLen == 0x06 && 
		//1Byte Lnge der Protokolladresse:Enthlt 4 fr 4 Byte Adressen 
		arp->ARP_PRLen == 0x04 &&
		//Ist das ARP Packet fr meine IP Addresse bestimmt
		//Vergleiche ARP Target IP Adresse mit meiner IP
		arp->ARP_TIPAddr == *((unsigned long *)&MYIP[0]))
	{
		printf("MYIP found, ");
		//check if frame is complete (it is reply to my own request)
		if(!memcmp(arp->ARP_THAddr, mymac, 6) && (arp->ARP_Op == 0x0200))
		{
			//it looks like reply to my own request
			ARP_Add_Entry(arp->ARP_SIPAddr, arp->ARP_SHAddr);
		}
		else if(arp->ARP_Op == 0x0100)
		{
			printf("replying\r");
			for(unsigned char a = 0; a < 6; a++)
			{
				//MAC Destadresse wird geschrieben mit MAC Sourceadresse
				ethernet->EnetPacketDest[a] = ethernet->EnetPacketSrc[a];
				//Meine MAC Adresse wird in Sourceadresse geschrieben
				ethernet->EnetPacketSrc[a] = mymac[a];
				//das gleiche fr die MAC Adresse
				//ARP MAC Targetadresse wird geschrieben mit ARP Sourceadresse
				arp->ARP_THAddr[a] = ethernet->EnetPacketDest[a];
				//ARP MAC Sourceadresse wird geschrieben mit My MAC Adresse
				arp->ARP_SHAddr[a] = mymac[a];
			}

			//ARP operation wird auf 2 gesetzt damit der andere merkt es ist ein ECHO	
			arp->ARP_Op = 0x0200;	
			//ARP Target IP Adresse wird geschrieben mit ARP Source IP Adresse 
			arp->ARP_TIPAddr = arp->ARP_SIPAddr;
			//Meine IP Adresse wird in ARP Source IP Adresse geschrieben
			arp->ARP_SIPAddr = *((unsigned long *)&MYIP[0]);

			//Nun ist das ARP buffer fertig zum Senden !!!
			//Sendet das erzeugte ARP Packet 
			Write_Ethernet_Frame (buffer,*bufferlen);
		}
	}
	return;
}

void SendArpRequest(const unsigned long ulRequestedIP)
{
	struct Ethernet_Header *ethernet;
	ethernet = (struct Ethernet_Header *)&packet[ETHER_OFFSET];
	ethernet->EnetPacketType = ARP_PROTOCOL_TYPE;

	struct ARP_Header *arp;
	arp = (struct ARP_Header *)&packet[ARP_OFFSET];

	//2 Byte Hardware Typ: Enthlt den Code fr Ethernet
	arp->ARP_HWType = 0x0100;

	//2 Byte Protokoll Typ: Enthlt den Code fr IP
	arp->ARP_PRType = 0x0008;

	//1Byte Lnge der Hardwareadresse:Enthlt 6 fr 6 Byte MAC Addresse
	arp->ARP_HWLen = 0x06;

	//1Byte Lnge der Protokolladresse:Enthlt 4 fr 4 Byte Adressen 
	arp->ARP_PRLen = 0x04;

	//Operation handelt es sich um eine anfrage
	arp->ARP_Op = 0x0100;

	//Ist das ARP Packet fr meine IP Addresse bestimmt
	//Vergleiche ARP Target IP Adresse mit meiner IP
	arp->ARP_TIPAddr = *((unsigned long *)&MYIP[0]);
	for(unsigned char a = 0; a < 6; a++)
	{
		//dest address: broadcast
		ethernet->EnetPacketDest[a] = 0xFF;
		ethernet->EnetPacketSrc[a] = mymac[a];
		arp->ARP_THAddr[a] = 0xFF;
		arp->ARP_SHAddr[a] = mymac[a];
	}

	arp->ARP_Op = 0x0100;	//ARP request
	
	printf("SendArpRequest: %d.%d.%d.%d\r",
		(uint8_t)(ulRequestedIP&0xFF),
		(uint8_t)((ulRequestedIP >> 8)&0xFF), 
		(uint8_t)((ulRequestedIP >> 16)&0xFF),
		(uint8_t)((ulRequestedIP >> 24)&0xFF));

	
	//ARP Target IP Adresse wird geschrieben mit ARP Source IP Adresse 
	arp->ARP_TIPAddr = ulRequestedIP;
	//Meine IP Adresse wird in ARP Source IP Adresse geschrieben
	arp->ARP_SIPAddr = *((unsigned long *)&MYIP[0]);

	//Nun ist das ARP buffer fertig zum Senden !!!
	//Sendet das erzeugte ARP Packet 
	//cli();
	Write_Ethernet_Frame (packet, ETHER_MIN_FRAME_SIZE);
	//sei();
}

/** Add entry to ARP cache table, delete oldest entry if all records already filled
	and entry is new
	\param IP IP address of host
	\param MAC MAC address of host
*/
void ARP_Add_Entry(uint32_t IP, char *MAC)
{
	printf("ARP add, ");
	static uint8_t u8_LastEntry = 0;
	for(uint8_t i=0; i<ARP_TABLE_SIZE; ++i)
	{
		if(arp_cache[i].IP == IP)
		{
		    printf ("updated MAC: %x.%x.%x.%x.%x.%x\n",MAC[0],MAC[1],MAC[2],
				MAC[3],MAC[4],MAC[5]);
			memcpy(arp_cache[i].MAC, MAC, 6);
			return;								//ARP cache updated
		}
	}
	arp_cache[u8_LastEntry].IP = IP;
    printf ("added MAC: %x.%x.%x.%x.%x.%x\n",MAC[0],MAC[1],MAC[2],
		MAC[3],MAC[4],MAC[5]);
	memcpy(arp_cache[u8_LastEntry].MAC, MAC, 6);
	if((++u8_LastEntry) >= ARP_TABLE_SIZE)
		u8_LastEntry = 0;
}

/** Find MAC in arp cache
	\param *MAC pointer to output buffer
	\return 0 if not found, 1 if found
*/
uint8_t ARP_Find(unsigned long IP, unsigned char *MAC)
{
	//cli();
	for(uint8_t i=0; i<ARP_TABLE_SIZE; i++)
	{
		if(arp_cache[i].IP == IP)
		{
			memcpy(MAC, arp_cache[i].MAC, 6);
			sei();
			return 1;								//ARP cache updated
		}
	}
	//sei();
	return 0;
}
